[comment {-*- tcl -*- doctools manpage}]
[comment {$Id: tclmilter.man,v 1.1.1.1 2005/06/26 22:03:02 muonics Exp $}]
[manpage_begin TclMilter n 1.0]
[copyright {2005 Michael Kirkham & Muonics}]
[moddesc {Tcl interface to SendMail's Milter API}]
[require Tcl 8.1]
[require TclMilter [opt 1.0]]

[description]
[para]
[uri "http://www.muonics.com/FreeStuff/TclMilter/" TclMilter] is a package
for Tcl (written in C) that implements an
interface to SendMail's Milter (Mail Filter) API for developing
custom scripted message rewriting and spam filtering processes. A
thread-enabled Tcl build is required due to Milter's threading
requirements. However, you can only run, configure, or register
callbacks for a milter from the first thread/interpreter the package
is loaded into. If you need to be able to stop the milter from
within Tcl, you can load the package in another thread to do so.
Only the "milter stop" command will function in secondary threads.

[para]
TclMilter follows libmilter's APIs very closely, with the following
exceptions for readability and flexibility:

[list_begin bullet]
[bullet]
The [cmd smfi_opensocket] command is not provided, only [cmd {milter main}]
as an interface to [cmd smfi_main].

[bullet]
The [cmd smfi_setconn], [cmd smfi_settimeout], [cmd smfi_setbacklog] and
[cmd smfi_setdbg] commands, as well as settings normally provided
through [cmd smfi_main]  are instead provided as [cmd {milter configure}]
options.

[bullet]
The [cmd smfi_register] command is provided as [cmd {milter register}];
however, unlike libmilter it's possible to register and unregister
callbacks at any time, rather than just once prior to calling [cmd main].

[bullet]
There isn't a single data storage value for [cmd smfi_setpriv]; rather,
TclMilter's [cmd {milter setpriv}] command uses a hash table so you can
set individual, custom variables with data to associate with a context.

[bullet]
The context provided to callbacks is not simply a pointer, but is
an actual Tcl command that can be used to interact with the context.

[list_end]

[section {RETURN CODES}]
[para]
Callbacks are invoked by TclMilter at various stages of message processing
in order to provide a filter with all of the information about the
MTA connection and message being sent.  These callbacks may return one
of five values to indicate to SendMail whether to continue processing
the message or whether to reject it for some reason.

[list_begin bullet]
[bullet]
[cmd SMFIS_CONTINUE] - Indicates that message processing should continue,
and that other callbacks should continue to be called (i.e., the message
is neither accepted or rejected yet).  Callbacks should always return
this value by default.

[bullet]
[cmd SMFIS_REJECT] - Indicates that the message should be rejected
immediately.  The milter may use [arg context] [cmd setreply] prior
to returning this value to indicate the status code and reason for the
rejection.

[bullet]
[cmd SMFIS_DISCARD] - Indicates that the message should be discared
immediately.

[bullet]
[cmd SMFIS_ACCEPT] - Indicates that the message should be accepted
as far as this milter is concerned.  No other callbacks will be
invoked.  However, the message may still be rejected for other
reasons - either by other milters or by SendMail itself.

[bullet]
[cmd SMFIS_TEMPFAIL] - Indicates that the message should be rejected
immediately with a temporary failure.  The milter may use [arg context]
[cmd setreply] prior to returning this value to indicate the status
code and reason for the rejection.

[list_end]

[section {MILTER CONTROL FUNCTIONS}]
[list_begin definitions]

[call [cmd {milter configure}] [opt "[arg option] [opt value]"] ...]
Configure Milter session parameters.  Given no options, returns a list
of option/value pairs indicating current milter configuration.  Given
an [arg option] and no values, returns the current value of the
specified option.  Otherwise, each option is set to the given value.

[list_begin opt]
[opt_def [option -name] [arg name]]
Specifies the name for the milter.  This should be a unique name among
all the milters being used by the MTA, and it must be set before calling
[cmd {milter main}] if at all.  The default is "TclMilter".

[opt_def [option -connection] [arg foo]]
Specifies the type of connection to use for communicating with SendMail,
as well as the appropriate parameters for the connection type.  It must be
set before calling [cmd {milter main}] if at all.  Per libmilter's
[uri "http://www.milter.org/milter_api/smfi_setconn.html" smfi_setconn]
documentation, the format is "proto:address":

[list_begin bullet]
[bullet]
{unix|local}:/path/to/file -- A named pipe.
[bullet]
inet:port@{hostname|ip-address} -- An IPV4 socket.
[bullet]
inet6:port@{hostname|ip-address} -- An IPV6 socket.
[list_end]

[nl]
The default is "unix:/var/run/tclmilter.sock" - a pipe socket using the
file [file /var/run/tclmilter.sock].

[opt_def [option -timeout] [arg seconds]]
Specifies the number of seconds that the Milter should wait for an MTA
connection before timing out. The timer can be reset using the [arg context]
[cmd progress] command.  This setting may only be changed
prior to calling [cmd {milter main}].  Default: 7210 seconds.

[opt_def [option -backlog] [arg count]]
Configures the number of incoming connections that the milter will
allow in the listen queue.  This setting may only be changed prior to
[cmd {milter main}].  The default for the milter is the system default.

[opt_def [option -debuglevel] [arg level]]
Configures the level of debug information that will be logged by libmilter.
This setting may be changed at any time.  Default: 0.

[list_end]

[nl]
[call [cmd {milter main}]]
Opens the socket to allow connection to SendMail and to begin filtering mail.
This command will not return unless [cmd {milter stop}] is called or the
milter receives a signal requiring a shutdown, such as SIGHUP.

[call [cmd {milter register}] [arg event] [opt [arg callback]]]
Register a callback for TclMilter to invoke at the specified stage on
mail processing.  Callbacks are supported corresponding to each of
those provided by libmilter its self.  Unlike libmilter, which must
have callbacks specified by the call to smfi_main(), callbacks for
TclMilter may be registered or unregistered at any time.
The [arg callback] argument specifies the name of the procedure
for TclMilter to invoke for the specified event.  If this argument
is not provided, the command will return the name of the currently
registered callback procedure for the specified event.

[nl]
The type and arguments that the callback receives depend on the
event, although every callback is provided with a unique context
ID that can be used to manipulate the message being processed.
Supported event types are as follows:

[list_begin definitions]

[call [cmd {milter register connect}] [arg connectProc]]
Arranges for proc [arg connectProc] to be called when a connection is
established to sendmail, before the exchange of HELO greetings.  The
callback is appended with three arguments: the context command,
the hostname and a list containing a dictionary of information from
the sockaddr stucture (e.g., address and port).

[nl]
proc [arg connectProc] { context hostname addrInfo } { ... }

[call [cmd {milter register helo}] [arg heloProc]]
Arranges for proc [arg heloProc] to be called when the client sends
its HELO greeting command.  The command is appended with two arguments:
the context and the hostname as provided in the HELO command.

[nl]
proc [arg heloProc] { context helohost } { ... }

[call [cmd {milter register envfrom}] [arg envfromProc]]
Arranges for proc [arg envfromProc] to be called when the client sends
the MAIL FROM command.  The command is appended with two elements:
the context command and a list of all of the addresses specified
in the MAIL FROM envelope.

[nl]
proc [arg envfromProc] { context fromList } { ... }

[call [cmd {milter register envrcpt}] [arg envrcptProc]]
Arranges for proc [arg envrcptProc] to be called when the client sends
the RCPT TO command.  The command is appended with two elements:
teh context command and a list of all of the addresses specified
in the RCPT TO envelope.

[nl]
proc [arg envrcptProc] { context rcptList } { ... }

[call [cmd {milter register header}] [arg headerProc]]
Arranges for proc [arg headerProc] to be called for each header sent
by the client.  The command is appended with three elements: the
context command, the header name and the contents of the header.

[nl]
proc [arg headerProc] { context name content } { ... }

[call [cmd {milter register eoh}] [arg eohProc]]
Arranges for proc [arg eohProc] to be called once all headers have
been sent and handled by the milter's "header" callback, before the
message body is sent.  The command is appended with one argument:
the context command.

[nl]
proc [arg eohProc] { context } { ... }

[call [cmd {milter register body}] [arg bodyProc]]
Arranges for proc [arg bodyProc] to be called for each block of
data in the message body.  The command is appended with two elements:
the context command and the body text.  Each message has only one
body, but this body may be sent in multiple blocks.  If the milter
intends to do content filtering on the message body, it is recommended
to use [arg context] [cmd setpriv] to build and store the complete
message body and perform content filtering in the [cmd eom] callback.

[nl]
proc [arg bodyProc] { context block } { ... }

[call [cmd {milter register eom}] [arg eomProc]]
Arranges for proc [arg eomProc] to be called once once all message
information has been received, including the entire message body,
and processed by the milter.  All message rewriting, such as adding
and removing headers or changing the body, must be performed
within this callback.  The command is appended with one element: the
context command.

[nl]
proc [arg eomProc] { context } { ... }

[call [cmd {milter register abort}] [arg abortProc]]
Arranges for proc [arg abortProc] to be called if message transmission
is aborted, such as due to a dropped connection.  This callback is not
invoked if message transmission is completed.  The command is appended
with one element: the context command.

[nl]
proc [arg abortProc] { context } { ... }

[call [cmd {milter register close}] [arg closeProc]]
Arranges for proc [arg closeProc] to be called when the client connection
has been closed.  This callback is not invoked if message transmission
is aborted.  The command is appended with one element: the context command.

[nl]
proc [arg closeProc] { context } { ... }

[list_end]
[nl]

[call [cmd {milter stop}]]
Stop the milter, causing [cmd {milter main}] to return.  Note that,
due to libmilter limitations, it is currently not possible to restart
the milter once stopped.  You will have to exit Tcl and then restart,
reload the package, etc. in order to start the milter up again.

[list_end]

[section {DATA ACCESS FUNCTIONS}]
[para]
The following context functions may be invoked from any callback to get
or store information about the message context or sendmail configuration.
Each context has a hash table associated with it, and a set of accessor
functions that work similar to Tcl's own [cmd set] command, allowing
the milter to associate data with each message that can be passed around
between callbacks and automatically garbage collected when the context
is deleted.

[list_begin definitions]

[call [arg context] [cmd getpriv] [opt [arg name]]]
If [arg name] is provided, this command returns the value from the hash
table with the given key [arg name].  If [arg name] is unspecified, a
list of [arg {name value}] pairs is returned with all of the private
data associated with the context, in dictionary format (i.e., suitable
for use with [cmd {array set}]).

[call [arg context] [cmd getsymval] [arg symbol]]
Returns the value for the specified symbol in the SendMail configuration.
The [arg symbol] may be either a single letter or a keyword surrounded
in {braces}.  Consult the libmilter and/or SendMail documentation for
more details.

[call [arg context] [cmd setpriv] [arg name] [opt [arg value]]]
Sets the specified value within the context's private hash table using
the given variable name/key [arg name].  If [arg value] is not specified,
returns the current value for the specified key, as with [cmd getpriv].

[call [arg context] [cmd setreply] [arg rcode] [opt "[opt [arg xcode]] [arg message]"]]
Sets the specific reply/status code for the message to be returned to the
client MTA and, optionally, the extended reply code and a message (such
as a reason for rejecting the message).

[call [arg context] [cmd unsetpriv] [opt [arg pattern]]]
This command could be considered a hybrid of Tcl's [cmd unset] and
[cmd {array unset}] commands.  It removes each variable whose name
matches the specified glob-style pattern from the context's private
data hash table.  If no pattern is specified then all private
data is deleted from the context.

[list_end]

[section {MESSAGE MODIFICATION FUNCTIONS}]
[para]
The following context functions may be invoked ONLY from the eom callback
to modify message contents.  The result of invoking these commands in
other callbacks is undefined.

[list_begin definitions]

[call [arg context] [cmd addheader] [arg name] [arg data]]
Adds a new header to the message with the given header [arg name] and contents [arg data].

[call [arg context] [cmd addrcpt] [arg rcpt]]
Instructs sendmail to deliver the message to the specified recipient
in addition to any others the message is already being sent to, provided
the message is not rejected.

[call [arg context] [cmd chgheader] [arg name] [opt [arg index]] [arg data]]
Changes an existing header in the message with the given header [arg name]
to the contents [arg data].  The [arg index] value specifies which
instance of the header is to be modified.  If unspecified, the first
instance of the header will be modified.  If no such header exists, it
will be added.

[call [arg context] [cmd delrcpt] [arg rcpt]]
Instructs sendmail NOT to deliver the message to specified recipient.  The
message may still be delivered to other recipients, provided the message
is not rejected.

[call [arg context] [cmd insheader] [arg name] [opt [arg index]] [arg data]]
Inserts a new header into the message with the given header [arg name]
and contents [arg data].  By default, headers are inserted
with [arg index] 0 - that is, they are inserted as the first header.
Otherwise, they are inserted before the [arg index]th header.  If
[arg index] is greater than the number of headers in the message,
the new header is added at the end, as with [cmd addheader].

[call [arg context] [cmd replacebody] [arg data]]
Replaces the entire message body with [arg data].  This can be useful,
for example, to remove questionable attachments or change message encoding.

[list_end]

[section {OTHER MESSAGE HANDLING FUNCTIONS}]
[para]
The following context functions may be invoked ONLY from the eom callback.
The result of invoking these commands in other callbacks is undefined.

[list_begin definitions]

[call [arg context] [cmd progress]]
This command may be called during long processes to inform SendMail/libmilter
that the message is still being processed.  When called, the timeout
clock is reset.

[call [arg context] [cmd quarantine] [arg reason]]
This command may be called to quarantine the message.  The message is
neither rejected nor delivered, but is left in the mail queue, requiring
administrator action to be delivered or unqueued.

[list_end]

[section EXAMPLE]
[example {
#!/usr/local/bin/tclsh
#
# A simple email address-based blacklist milter
#

package require Tcl 8.4
package require TclMilter 1.0

# Configuration

set blacklist {
    friend@public.com
    spammer@spamdomain.com
}

milter configure -name Blacklist
milter configure -connection {unix:/var/run/blmilter.sock}

# Check to see if an address is on the blacklist

proc is_blacklisted { address } {
    set address [string trim $address <>]

    if {[lsearch -exact $::blacklist $address] != -1} {
        return 1
    }

    return 0
}

# Callbacks

proc bl_header { context name data } {
    if {($name == "From") && [is_blacklisted $data]} {
        return SMFIS_REJECT
    }

    return SMFIS_CONTINUE
}

proc bl_envfrom { context addressList } {
    foreach address $addressList {
        if {[is_blacklisted $address]} {
            return SMFIS_REJECT
        }
    }

    return SMFIS_CONTINUE
}

# Register the callbacks and start up the filter

milter register header bl_header
milter register envfrom bl_envfrom
milter main
file delete /var/run/blmilter.sock
}]

[section LICENSE]
TclMilter is provided under the GNU General Public License
(GPL) (see the file [file license.terms] for details).

[see_also [uri "http://www.milter.org/milter_api/api.html" "Milter API"]]
[see_also [uri "http://www.muonics.com/rfc/rfc821.php" "RFC 821"]]
[see_also [uri "http://www.muonics.com/rfc/rfc2821.php" "RFC 2821"]]
[see_also [uri "http://www.muonics.com/rfc/rfc1893.php" "RFC 1893"]]
[see_also [uri "http://www.muonics.com/rfc/rfc2034.php" "RFC 2034"]]

[keywords milter tcl email sendmail spam filter]

[section ACKNOWLEDGEMENTS]
Portions of this document, as well as the package its self, are based on the
Milter API specification.

[manpage_end]
